
import sys
import os
import time
sys.path.append(os.getcwd())
from cdplib.log import Log
log = Log("Simplified Process Model")


class SimplifiedProcessModel:

    def __init__(self):
        self._log = Log("Simplified Process Model")

    @property
    def process_stages(self):
        '''
        For machine learning predictions we divide the process into
        big stages, stages can be skipped dependeing on the IHS of the
        wheelset. We use all information geathered during the previous
        process stages to make predictions for the next stage.
        '''
        import networkx as nx
        '''
        critical_stations = {"A": [421, 110]}

        critical_stations["B"] = [130]

        critical_stations["C"] = [140, 150]

        critical_stations["D"] = [410]

        critical_stations["E"] = [510, 520, 535,
                                530, 550]

        critical_stations["F"] = [490, 480, 430, 170]

        critical_stations["G"] = [595, 190, 630]

        critical_stations["H"] = [640]

        critical_stations["I"] = [650, 560]

        critical_stations["J"] = [675]

        critical_stations["K"] = [690]

        critical_stations["L"] = [710, 670]

        stages_graph = nx.DiGraph()

        for stage in critical_stations:
            stages_graph.add_node(stage, stations=critical_stations[stage])

        stages_graph.add_edge("A", "B")
        stages_graph.add_edge("B", "C")
        stages_graph.add_edge("C", "D")
        stages_graph.add_edge("D", "E")
        stages_graph.add_edge("D", "F")
        stages_graph.add_edge("E", "G")
        stages_graph.add_edge("F", "G")
        stages_graph.add_edge("G", "H")
        stages_graph.add_edge("H", "I")
        stages_graph.add_edge("I", "J")
        stages_graph.add_edge("J", "K")
        stages_graph.add_edge("K", "L")
        '''

        critical_stations = {"A": [421, 110]}

        critical_stations["B"] = [130]

        critical_stations["C"] = [140, 150]

        critical_stations["D"] = [410]

        critical_stations["E"] = [510, 520, 535,
                                530, 320, 550]

        #critical_stations["F"] = [490, 480, 430, 595]

        #critical_stations["G"] = [170, 506, 430]

        # Merge the two above since they both contain station 430
        critical_stations["F"] = [490, 480, 430, 595, 170, 506, 430]

        critical_stations["H"] = [190]

        critical_stations["I"] = [320, 340, 630]

        #This path is in parallel with the rest of the paths. handle as an exception
        #critical_stations["J"] = [680]

        critical_stations["K"] = [640]

        stages_graph = nx.DiGraph()
        for stage in critical_stations:
            stages_graph.add_node(node_for_adding=stage, stations=critical_stations[stage])

        stages_graph.add_edge("A", "B")

        stages_graph.add_edge("B", "C")
        stages_graph.add_edge("B", "K")
        #stages_graph.add_edge("B", "J")

        stages_graph.add_edge("C", "D")

        stages_graph.add_edge("D", "E")
        stages_graph.add_edge("D", "F")
        #stages_graph.add_edge("D", "G")
        stages_graph.add_edge("D", "H")

        stages_graph.add_edge("E", "H")

        #stages_graph.add_edge("F", "G")

        #stages_graph.add_edge("G", "H")

        stages_graph.add_edge("E", "I")
        stages_graph.add_edge("H", "I")

        #stages_graph.add_edge("J", "K")

        stages_graph.add_edge("I", "K")

        return stages_graph

    def process_stages_paths(self):
        import networkx as nx
        result_graph = self.process_stages

        result_dict = {}
        stage_list = result_graph.nodes()
        for stage in stage_list:
            for path in nx.all_simple_paths(result_graph, source="A", target=stage):
                station_list = []
                for stage in path:
                        for station in result_graph.nodes()[stage]['stations']:
                            station_list.append(station)
                result_dict[str(path)] = station_list
        return result_dict

    def get_stage(self, station_nummer: int):
        import networkx as nx
        result_graph = self.process_stages
        for path in nx.all_simple_paths(result_graph, source="A", target="K"):
            station_list = []
            for stage in path:
                    for station in result_graph.nodes()[stage]['stations']:
                        if station == station_nummer:
                            return stage
        return

    def calculate_stage_path(self, stations_list: list):
        position_list = list(self.process_stages.nodes())
        current_index = 0
        stage_path = []
        for station in stations_list:
            stage = self.get_stage(station)
            if stage is not None:
                new_index = position_list.index(stage)
                if current_index <= new_index:
                    if stage not in stage_path:
                        stage_path.append(stage)
                        current_index = new_index
                else:
                    self._log.warning('Path does not conform to the model, no prediction can be made')
                    return
        return str(stage_path)

    def get_stations_for_prediction_model(self, stations_list):
        stages_paths = self.process_stages_paths()
        prediction_stations = stages_paths[self.calculate_stage_path(stations_list)]

        # Hack solution for handling that station 680 is running in parallel with most other stations
        if len(prediction_stations) > 3:
            prediction_stations.append(680)
        return prediction_stations
